"""
    CUSTOM READERS CLASSES
        - Class which manages reader tasks like auth, requests, pagination
"""
import os

import yaml
from sailthru import SailthruClient

from sdc_dp_helpers.api_utilities.retry_managers import request_handler
from sdc_dp_helpers.sailthru.config_magagers import ConfigManager, get_queries
from sdc_dp_helpers.api_utilities.date_managers import date_handler


class CustomSailThruReader:
    """
        Custom SailThru Reader
    """

    def __init__(self, api_key_path):
        self.api_key_path = api_key_path
        self.sailthru_client = SailthruClient(
            api_key=self.get_api_key()[0],
            secret=self.get_api_key()[1]
        )

    def get_api_key(self) -> list:
        """
            Gathers key and value pairs of key and their secret.
        """
        with open(self.api_key_path, 'r') as file:
            data = yaml.safe_load(file)
            return [data.get('api_key'), data.get('api_secret')]

    def get_all_templates(self, filter_by='name') -> list:
        """
            Templates are dynamic for the SailThru stats endpoint.
            Therefore we can gather all templates for each api request and return
            a list of relevant information for the reader.
            :filter_by: str. The key value that we can return, such as name or id.
            :return: list. A list of relevant template values.
        """
        response = self.sailthru_client.get_templates()
        data = response.get_body()
        print(f'Template Response:{data}')

        items = []
        for item in data['templates']:
            items.append(item.get(filter_by, None))

        return items

    @request_handler(
        wait=int(os.environ.get('REQUEST_WAIT_TIME', 0)),
        backoff_factor=float(os.environ.get('REQUEST_BACKOFF_FACTOR', 0.01)),
        backoff_method=os.environ.get('REQUEST_BACKOFF_METHOD', 0.01)
    )
    def get_all_stats(self, configs):
        """
        Stats returned are based on the cred provided &
        Additional parameters are dependent on the stat value:
            - list - information about your subscriber counts on
              all lists or a particular list
            - blast - information about a particular campaign
            - send - information about a particular triggered message

        :configs: str. Path to the list of configs to iterate over.
        :returns: dict. Response of the particular query.
        """

        query_config = get_queries(configs)
        print('Query config:{}'.format(query_config))

        data_set = []
        for query in query_config:
            # handle date ranges

            query['start_date'] = date_handler(query['start_date'])
            query['end_date'] = date_handler(query['end_date'])
            print(f'Query: {query}')

            # send queries require templates, here we use dynamic templates
            api_get = ConfigManager(sailthru_client=self.sailthru_client, query=query)

            if query.get('stat') == 'send':
                templates = self.get_all_templates()

                for template in templates:
                    query.update({'template': template})
                    data = api_get.get_stat_api()

                    if data is not None:
                        data_set.append(data)

            else:
                data = api_get.get_stat_api()
                if data is not None:
                    data_set.append(data)

        return data_set



"""
    Utils functions
"""
import yaml


def init_config(file_name: str = './example.yaml'):
    """
        Util method that creates an example config at the given file_name.
        :file_name: Full path to config YAML file.
    """
    config = {
        'query_config': [{
            'action': 'stats',
            'data': {
                'stat': 'send',
                'template': 'alert'}
        },
            {'action': 'stats',
             'data': {
                 'stat': 'send',
                 'template': 'alert subscribers'}
             }
        ]}

    # write to file
    with open(file_name, 'w') as file_:
        yaml.dump(config, file_)



import warnings

import yaml


def get_queries(configs):
    """
        Gathers query configurations.yml file data.
    """
    with open(configs, 'r') as file:
        query_config = yaml.safe_load(file)
    return query_config.get('query_config')


class ConfigManager:
    """
        SailThru Config Manager
    """
    def __init__(self, sailthru_client, query):
        self.sailthru_client = sailthru_client
        self.query = query

    def get_stat_api(self):
        """
            Request various stats from Sailthru about primary list
            membership or campaign and triggered message activity.
            Endpoint URL: https://api.sailthru.com/stats

            :return: If there is no relevant data, none is
                     returned and reason is printed.
        """
        response = self.sailthru_client.api_get(
            action='stats',
            data=self.query
        )

        # fatal false negative is raised, this needs to be handled
        if not response.is_ok():
            error = response.get_error()
            message = 'APIError:{}, Status Code:{}, Error Code:{}'.format(
                error.get_message(),
                str(response.get_status_code()),
                str(error.get_error_code()))
            if (str(response.get_status_code()) != '404'
                    and str(error.get_error_code()) != '99'):
                raise RuntimeError(message)
            else:
                warnings.warn(message)

        elif response.is_ok():
            response_data = response.get_body()
            response_data.update({
                'stat': self.query.get('stat'),
                'template': self.query.get('template')
            })

            print(f'Response Data: {response_data}')
            return response_data
        else:
            return None



from datetime import datetime, timedelta


def date_handler(date_string, date_format='%Y-%m-%d'):
    """
    Takes a date string or phrase and returns the valid date in
    the specified format.
    :date_string: A phrase like "3_days_ago", "yesterday" or "today"
                     or a date string such as "2021-01-01".
    :date_format: The returned formatting of the date.
    """
    try:
        if date_string == 'today':
            return datetime.now().strftime(date_format)
        if date_string == 'yesterday':
            return (datetime.now() - timedelta(days=1)).strftime(date_format)
        if '_days_ago' in date_string:
            return phrase_to_date(phrase=date_string, date_format=date_format)
        return date_string
    except ValueError as err:
        raise ValueError('StartDate requires a valid input '
                         'such as "today", "yesterday" or "<int>_days_ago".') from err


def phrase_to_date(phrase, date_format='%Y-%m-%d'):
    """
    Takes a typical phrase such as "3_days_ago" and returns a
    date based on that phrase.
    :phrase: str. Something like 3_days_ago or 25_days_ago etc.
    :return: '%Y-%m-%d'
    """
    try:
        date_delta = int(phrase.split('_')[0])
        return (datetime.now() - timedelta(days=date_delta)).strftime(date_format)
    except ValueError as err:
        raise ValueError(f'Phrasing for date: {phrase} is not valid, '
                         f'try something like: 3_days_ago') from err
